@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id$ */
+
 /* Command list document- and page-level code. */
 #include "memory_.h"
 #include "string_.h"
@@ -30,6 +32,10 @@
 #include "gsicc_cache.h"
 #include "gxdevsop.h"
 
+#ifdef PACIFY_VALGRIND
+#include <valgrind/helgrind.h>
+#endif
+
 extern dev_proc_open_device(pattern_clist_open_device);
 
 /* GC information */
@@ -181,7 +187,7 @@ const gx_device_procs gs_clist_device_procs = {
     gx_forward_encode_color,
     gx_forward_decode_color,
     NULL,                       /* pattern_manage */
-    gx_default_fill_rectangle_hl_color,
+    clist_fill_rectangle_hl_color,
     gx_default_include_color_space,
     gx_default_fill_linear_color_scanline,
     clist_fill_linear_color_trapezoid,
@@ -196,7 +202,9 @@ const gx_device_procs gs_clist_device_procs = {
     clist_copy_planes,         /* copy planes */
     gx_default_get_profile,
     gx_default_set_graphics_type_tag,
-    clist_strip_copy_rop2
+    clist_strip_copy_rop2,
+    clist_strip_tile_rect_devn,
+    clist_copy_alpha_hl_color,
 };
 
 /*------------------- Choose the implementation -----------------------
@@ -211,6 +219,12 @@ const clist_io_procs_t *clist_io_procs_memory_global = NULL;
 void
 clist_init_io_procs(gx_device_clist *pclist_dev, bool in_memory)
 {
+#ifdef PACIFY_VALGRIND
+    VALGRIND_HG_DISABLE_CHECKING(&clist_io_procs_file_global, sizeof(clist_io_procs_file_global));
+    VALGRIND_HG_DISABLE_CHECKING(&clist_io_procs_memory_global, sizeof(clist_io_procs_memory_global));
+#endif
+    /* if clist_io_procs_file_global is NULL, then BAND_LIST_STORAGE=memory */
+    /* was specified in the build, and "file" is not available */
     if (in_memory || clist_io_procs_file_global == NULL)
         pclist_dev->common.page_info.io_procs = clist_io_procs_memory_global;
     else
@@ -329,8 +343,8 @@ clist_init_bands(gx_device * dev, gx_device_memory *bdev, uint data_size,
     cdev->nbands = nbands;
 #ifdef DEBUG
     if (gs_debug_c('l') | gs_debug_c(':'))
-        dlprintf4("[:]width=%d, band_width=%d, band_height=%d, nbands=%d\n",
-                  bdev->width, band_width, band_height, nbands);
+        dmlprintf4(dev->memory, "[:]width=%d, band_width=%d, band_height=%d, nbands=%d\n",
+                   bdev->width, band_width, band_height, nbands);
 #endif
     return 0;
 }
@@ -402,7 +416,9 @@ clist_init_data(gx_device * dev, byte * init_data, uint data_size)
     cdev->graphics_type_tag = target->graphics_type_tag;	/* initialize to same as target */
 
     /* Call create_buf_device to get the memory planarity set up. */
-    cdev->buf_procs.create_buf_device(&pbdev, target, 0, NULL, NULL, clist_get_band_complexity(0, 0));
+    code = cdev->buf_procs.create_buf_device(&pbdev, target, 0, NULL, NULL, clist_get_band_complexity(0, 0));
+    if (code < 0)
+        return code;
     /* HACK - if the buffer device can't do copy_alpha, disallow */
     /* copy_alpha in the commmand list device as well. */
     if (dev_proc(pbdev, copy_alpha) == gx_no_copy_alpha)
@@ -614,8 +630,8 @@ clist_reset_page(gx_device_clist_writer *cwdev)
     cwdev->page_bfile_end_pos = 0;
     /* Indicate that the colors_used information hasn't been computed. */
     cwdev->page_info.scan_lines_per_colors_used = 0;
-    memset(cwdev->page_info.band_colors_used, 0,
-           sizeof(cwdev->page_info.band_colors_used));
+    memset(cwdev->page_info.band_color_usage, 0,
+           sizeof(cwdev->page_info.band_color_usage));
 }
 
 /* Open the device's bandfiles */
@@ -685,11 +701,13 @@ static void
 clist_set_planar(gx_device *dev)
 {
     gx_device_clist_common * cdev = &((gx_device_clist *)dev)->common;
+    int ret;
 
-    if (dev_proc(dev, dev_spec_op)(dev, gxdso_is_native_planar, NULL, 0) > 0) {
-        cdev->is_planar = true;
+    ret = dev_proc(dev, dev_spec_op)(dev, gxdso_is_native_planar, NULL, 0);
+    if (ret > 0) {
+        cdev->is_planar = ret;
     } else {
-        cdev->is_planar = false;
+        cdev->is_planar = 0;
     }
 }
 
@@ -820,7 +838,7 @@ clist_end_page(gx_device_clist_writer * cldev)
             code = 0;
     }
     if (code >= 0) {
-        clist_compute_colors_used(cldev);
+        clist_compute_color_usage(cldev);
         ecode |= code;
         cldev->page_bfile_end_pos = cldev->page_info.io_procs->ftell(cldev->page_bfile);
     }
@@ -836,10 +854,10 @@ clist_end_page(gx_device_clist_writer * cldev)
 #ifdef DEBUG
     if (gs_debug_c('l') | gs_debug_c(':')) {
         if (cb.pos <= 0xFFFFFFFF)
-            dlprintf2("[:]clist_end_page at cfile=%lu, bfile=%lu\n",
+            dmlprintf2(cldev->memory, "[:]clist_end_page at cfile=%lu, bfile=%lu\n",
                   (unsigned long)cb.pos, (unsigned long)cldev->page_bfile_end_pos);
         else
-            dlprintf3("[:]clist_end_page at cfile=%lu%0lu, bfile=%lu\n",
+            dmlprintf3(cldev->memory, "[:]clist_end_page at cfile=%lu%0lu, bfile=%lu\n",
                 (unsigned long) (cb.pos >> 32), (unsigned long) (cb.pos & 0xFFFFFFFF),
                 (unsigned long)cldev->page_bfile_end_pos);
     }
@@ -847,6 +865,20 @@ clist_end_page(gx_device_clist_writer * cldev)
     return 0;
 }
 
+gx_color_usage_bits
+gx_color_index2usage(gx_device *dev, gx_color_index color)
+{
+    gx_color_usage_bits bits = 0;
+    int i;
+
+    for (i = 0; i < dev->color_info.num_components; i++) {
+        if (color & dev->color_info.comp_mask[i])
+            bits |= (1<<i);
+    }
+
+    return bits;
+}
+
 /* Compute the set of used colors in the page_info structure.
  *
  * NB: Area for improvement, move states[band] and page_info to clist
@@ -855,7 +887,7 @@ clist_end_page(gx_device_clist_writer * cldev)
  */
 
 void
-clist_compute_colors_used(gx_device_clist_writer *cldev)
+clist_compute_color_usage(gx_device_clist_writer *cldev)
 {
     int nbands = cldev->nbands;
     int bands_per_colors_used =
@@ -865,15 +897,15 @@ clist_compute_colors_used(gx_device_clist_writer *cldev)
 
     cldev->page_info.scan_lines_per_colors_used =
         cldev->page_band_height * bands_per_colors_used;
-    memset(cldev->page_info.band_colors_used, 0,
-           sizeof(cldev->page_info.band_colors_used));
+    memset(cldev->page_info.band_color_usage, 0,
+           sizeof(cldev->page_info.band_color_usage));
     for (band = 0; band < nbands; ++band) {
         int entry = band / bands_per_colors_used;
 
-        cldev->page_info.band_colors_used[entry].or |=
-            cldev->states[band].colors_used.or;
-        cldev->page_info.band_colors_used[entry].slow_rop |=
-            cldev->states[band].colors_used.slow_rop;
+        cldev->page_info.band_color_usage[entry].or |=
+            cldev->states[band].color_usage.or;
+        cldev->page_info.band_color_usage[entry].slow_rop |=
+            cldev->states[band].color_usage.slow_rop;
 
     }
 }
@@ -906,7 +938,7 @@ clist_VMerror_recover(gx_device_clist_writer *cldev,
         }
     } while (pages_remain);
 
-    if_debug1('L', "[L]soft flush of command list, status: %d\n", code);
+    if_debug1m('L', cldev->memory, "[L]soft flush of command list, status: %d\n", code);
     return code;
 }
 
@@ -946,7 +978,7 @@ clist_VMerror_recover_flush(gx_device_clist_writer *cldev,
     }
 
     code = (reset_code < 0 ? reset_code : free_code < 0 ? old_error_code : 0);
-    if_debug1('L', "[L]hard flush of command list, status: %d\n", code);
+    if_debug1m('L', cldev->memory, "[L]hard flush of command list, status: %d\n", code);
     return code;
 }
 
@@ -1068,12 +1100,17 @@ clist_icc_writetable(gx_device_clist_writer *cldev)
     clist_icctable_entry_t *curr_entry;
     int size_data;
     int k;
+    bool rend_is_valid;
 
     /* First we need to write out the ICC profiles themselves and update
-       in the table where they will be stored and their size. */
+       in the table where they will be stored and their size.  Set the
+       rend cond valid flag prior to writing */
     curr_entry = icc_table->head;
     for ( k = 0; k < number_entries; k++ ){
+        rend_is_valid = curr_entry->icc_profile->rend_is_valid;
+        curr_entry->icc_profile->rend_is_valid = curr_entry->render_is_valid;
         curr_entry->serial_data.file_position = clist_icc_addprofile(cldev, curr_entry->icc_profile, &size_data);
+        curr_entry->icc_profile->rend_is_valid = rend_is_valid;
         curr_entry->serial_data.size = size_data;
         rc_decrement(curr_entry->icc_profile, "clist_icc_writetable");
         curr_entry->icc_profile = NULL;
@@ -1108,7 +1145,11 @@ clist_icc_addprofile(gx_device_clist_writer *cldev, cmm_profile_t *iccprofile, i
 
     clist_file_ptr cfile = cldev->page_cfile;
     int64_t fileposit;
+#if defined(DEBUG) || defined(PACIFY_VALGRIND)
+    gsicc_serialized_profile_t profile_data = { 0 };
+#else
     gsicc_serialized_profile_t profile_data;
+#endif
     int count1, count2;
 
     /* Get the current position */
@@ -1116,8 +1157,8 @@ clist_icc_addprofile(gx_device_clist_writer *cldev, cmm_profile_t *iccprofile, i
     /* Get the serialized header */
     gsicc_profile_serialize(&profile_data, iccprofile);
     /* Write the header */
-    if_debug1('l', "[l]writing icc profile in cfile at pos %ld\n",fileposit);
-    count1 = cldev->page_info.io_procs->fwrite_chars(&profile_data, sizeof(gsicc_serialized_profile_t), cfile);
+    if_debug1m('l', cldev->memory, "[l]writing icc profile in cfile at pos %"PRId64"\n",fileposit);
+    count1 = cldev->page_info.io_procs->fwrite_chars(&profile_data, GSICC_SERIALIZED_SIZE, cfile);
     /* Now write the profile */
     count2 = cldev->page_info.io_procs->fwrite_chars(iccprofile->buffer, iccprofile->buffer_size, cfile);
     /* Return where we wrote this in the cfile */
@@ -1157,6 +1198,7 @@ clist_icc_addentry(gx_device_clist_writer *cdev, int64_t hashcode_in, cmm_profil
         entry->serial_data.size = -1;
         entry->serial_data.file_position = -1;
         entry->icc_profile = icc_profile;
+        entry->render_is_valid = icc_profile->rend_is_valid;
         rc_increment(icc_profile);
         icc_table = gs_alloc_struct(stable_mem, clist_icctable_t, 
                                     &st_clist_icctable, "clist_icc_addentry");
@@ -1173,9 +1215,9 @@ clist_icc_addentry(gx_device_clist_writer *cdev, int64_t hashcode_in, cmm_profil
     } else {
         /* First check if we already have this entry */
         curr_entry = icc_table->head;
-        for ( k = 0; k < icc_table->tablesize; k++ ) {
-            if ( curr_entry->serial_data.hashcode == hashcode )
-                return(0);  /* A hit */
+        for (k = 0; k < icc_table->tablesize; k++) {
+            if (curr_entry->serial_data.hashcode == hashcode)
+                return 0;  /* A hit */
             curr_entry = curr_entry->next;
         }
          /* Add a new ICC profile */
@@ -1191,6 +1233,7 @@ clist_icc_addentry(gx_device_clist_writer *cdev, int64_t hashcode_in, cmm_profil
         entry->serial_data.size = -1;
         entry->serial_data.file_position = -1;
         entry->icc_profile = icc_profile;
+        entry->render_is_valid = icc_profile->rend_is_valid;
         rc_increment(icc_profile);
         icc_table->final->next = entry;
         icc_table->final = entry;
@@ -1208,7 +1251,7 @@ clist_writer_push_no_cropping(gx_device_clist_writer *cdev)
 
     if (buf == NULL)
         return_error(gs_error_VMerror);
-    if_debug1('v', "[v]push cropping[%d]\n", cdev->cropping_level);
+    if_debug1m('v', cdev->memory, "[v]push cropping[%d]\n", cdev->cropping_level);
     buf->next = cdev->cropping_stack;
     cdev->cropping_stack = buf;
     buf->cropping_min = cdev->cropping_min;
@@ -1244,7 +1287,7 @@ clist_writer_pop_cropping(gx_device_clist_writer *cdev)
     cdev->temp_mask_id = buf->temp_mask_id;
     cdev->cropping_stack = buf->next;
     cdev->cropping_level--;
-    if_debug1('v', "[v]pop cropping[%d]\n", cdev->cropping_level);
+    if_debug1m('v', cdev->memory, "[v]pop cropping[%d]\n", cdev->cropping_level);
     gs_free_object(cdev->memory, buf, "clist_writer_transparency_pop");
     return 0;
 }
@@ -1253,7 +1296,7 @@ int
 clist_writer_check_empty_cropping_stack(gx_device_clist_writer *cdev)
 {
     if (cdev->cropping_stack != NULL) {
-        if_debug1('v', "[v]Error: left %d cropping(s)\n", cdev->cropping_level);
+        if_debug1m('v', cdev->memory, "[v]Error: left %d cropping(s)\n", cdev->cropping_level);
         return_error(gs_error_unregistered); /* Must not happen */
     }
     return 0;
